#include <stdio.h>

#include "ExceptionComponent.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
// The infrastructure code relies on your having assigned appropriate values to these global variables.
// All are just strings, used only for informational purposes (i.e., you can fill in anything you want).
///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////

const char* PACKAGE_NAME = "Exception Component Package";
const char* PACKAGE_VERSION = "V1.0";
const char* PACKAGE_DATE = "February 12, 2006";
const char* PACKAGE_AUTHOR = "John Zahorjan";

///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
// You must also fill in this table, one entry per component class you have defined.
// Each entry contains {name, CreateObject() function} for the components of the
//      corresponding type in this package.
//  This example package defines three component types.
///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////

ComponentDescriptor	componentTypes[] = {
	{"ExceptionComponent",	ExceptionComponent::CreateObject},

	{0, 0}	// you MUST terminate the list with this entry
};

///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////
// ExceptionComponent - the first of two component types defined by this package
///////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////

	// By convention, we put descriptions of the in/out-put ports for this component
	// type in class static arrays.  Each array MUST BE TERMINATED with NO_XXPORT entry.

	INPORTDESC	ExceptionComponent::INPORT_DESCRIPTION[] = {
		{STD_INPORT, "ExampleInput", 32, 0},		// entries are <input port type>, <name>, <width, in bits>, <isClockdownInput>

		{NO_INPORT, "", 0, 0}			// you must end the list with a NO_INPORT entry!
	};

	OUTPORTDESC	ExceptionComponent::OUTPORT_DESCRIPTION[] = {
		{COMBINATIONAL_OUTPORT, "ExampleOutput", 3},

		{NO_OUTPORT, "", 0}			// you must end the list with a NO_OUTPORT entry!
	};

	CUserDefComponentBase* ExceptionComponent::CreateObject()
	{
		return new ExceptionComponent;
	}

	ExceptionComponent::ExceptionComponent()
	{
		PrintMsg("Created ExceptionComponent\n");
	}

	ExceptionComponent::~ExceptionComponent()
	{
		PrintMsg( "ExceptionComponent destroyed\n" );	// this printf is just an example (to show you can use printf())
	}

	void ExceptionComponent::GetPortDescriptionVectors( INPORTDESC** pInputPorts, OUTPORTDESC** pOutputPorts )
	{
		*pInputPorts = INPORT_DESCRIPTION;
		*pOutputPorts = OUTPORT_DESCRIPTION;
	}

    // This is the only routine that should need any modification (although some data
    // items above, that define the input and output ports, need modifying as well).
    // This routine will be called once per cycle, at clock up.
    // However, to function correctly, the output values you compute MUST BE pure functions
    // of the input values -- you cannot expect code that saves any kind of information
    // between invocations, and uses it to compute outputs, to work correctly.

    // The parameters pInputArray and pOutputArray are arrays of int's.
    // This sample does not -- just makes its output eqal its input.
	HALT_STATUS ExceptionComponent::ComputeOutputs(int* pInputArray, int* pOutputArray)
	{
        pOutputArray[0] = pInputArray[0];

		return SMOK_CONTINUE;
	}

	HALT_STATUS ExceptionComponent::LatchInputs(int* pInputArray)
	{
		// this component type is purely combinational, so has nothing to do here
		return SMOK_CONTINUE;
	}

	bool ExceptionComponent::Reinitialize()
	{
		return true;	// nothing to do, because this component type has no state
	}

	void ExceptionComponent::PrintMsg(const char* s)
	{
		// This output will show up in the SMOK trace output window, if there is one.
		// If there isn't one, the output won't show up anywhere.
		printf(s);
	}



