#ifndef PACKAGEBASE_H
#define PACKAGEBASE_H

#include "DLLInterface.h"

//////////////////////////////////////////////////////////////////////////
// Require user to provide information on their package
//////////////////////////////////////////////////////////////////////////

extern const char* PACKAGE_VERSION;
extern const char* PACKAGE_DATE;
extern const char* PACKAGE_AUTHOR;

///////////////////////////////////////////////////////////////
// This is the base class of all user defined component types
///////////////////////////////////////////////////////////////

class CUserDefComponentBase 
{
	///////////////////////////////////////////////////////////////////
	// YOU MUST IMPLEMENT THE FOLLOWING IN YOUR DERIVED CLASS(ES)
	///////////////////////////////////////////////////////////////////
	public:
		virtual ~CUserDefComponentBase();

		virtual void			GetPortDescriptionVectors( INPORTDESC** pInputPorts, OUTPORTDESC** pOutputPorts ) = 0;
		virtual HALT_STATUS		ComputeOutputs(int* pInputArray, int* pOutputArray) = 0;
		virtual HALT_STATUS		LatchInputs(int* pInputArray) = 0;
		virtual bool			Reinitialize() = 0;

		//////////////////////////////////////////////////////////////////
		// Each class you derive MUST ALSO HAVE a static function 
		// (that new's up an object of your class and returns it).
		// That function is of type CREATE_DEFINED_COMPONENT_PROC_TYPE.
		// It is entered into the componentTypes[] array defined at
		// the end of this file.
		//////////////////////////////////////////////////////////////////

	///////////////////////////////////////////////////////////////////
	// Methods implemented by the infrastructure
	///////////////////////////////////////////////////////////////////
	public:
		void	RegisterCallbacks( void* pSMOKObj, READMEMPORT_PROC pReadMemProc, WRITEMEMPORT_PROC pWriteMemProc );
	protected:
		int		ReadMemory ( int inputPortNum, int address, int numBytes );
		void	WriteMemory( int inputPortNum, int address, int data, int numBytes );

	///////////////////////////////////////////////////////////////////
	// Instance variables for infrastructure use only
	///////////////////////////////////////////////////////////////////
	private:
		void*				m_pSMOKObj;
		READMEMPORT_PROC	m_pReadMemProc;
		WRITEMEMPORT_PROC	m_pWriteMemProc;
};

///////////////////////////////////////////////////////////////
// Defines the type of a table that you need to fill in.
// The table has a row for each type defined by your package.
// Each row contains the name of that type plus a pointer to
//		a function that can create an object of that type.
///////////////////////////////////////////////////////////////

typedef struct {
	const char*							componentName;
	CREATE_DEFINED_COMPONENT_PROC_TYPE	pCreateObjectFunction;
} ComponentDescriptor;

extern ComponentDescriptor componentTypes[];	// this is the package description table you need to create

#endif // PACKAGE_BASE